@@ -1,13 +1,16 @@
 import sys
 from functools import lru_cache
+from marshal import loads as marshal_loads, dumps as marshal_dumps
 from random import randint
 from time import time
-from typing import Any, Dict, Iterable, List, Optional, Type, Union
+from typing import Any, cast, Dict, Iterable, List, Optional, Type, Union
 
 from . import errors
 from .color import Color, ColorParseError, ColorSystem, blend_rgb
+from .repr import rich_repr, RichReprResult
 from .terminal_theme import DEFAULT_TERMINAL_THEME, TerminalTheme
 
+
 # Style instances and style definitions are often interchangeable
 StyleType = Union[str, "Style"]
 
@@ -26,6 +29,7 @@ class _Bit:
         return None
 
 
+@rich_repr
 class Style:
     """A terminal style.
 
@@ -59,6 +63,7 @@ class Style:
     _set_attributes: int
     _hash: int
     _null: bool
+    _meta: Optional[bytes]
 
     __slots__ = [
         "_color",
@@ -71,6 +76,7 @@ class Style:
         "_style_definition",
         "_hash",
         "_null",
+        "_meta",
     ]
 
     # maps bits on to SGR parameter
@@ -109,6 +115,7 @@ class Style:
         encircle: Optional[bool] = None,
         overline: Optional[bool] = None,
         link: Optional[str] = None,
+        meta: Optional[Dict[str, Any]] = None,
     ):
         self._ansi: Optional[str] = None
         self._style_definition: Optional[str] = None
@@ -159,6 +166,7 @@ class Style:
 
         self._link = link
         self._link_id = f"{time()}-{randint(0, 999999)}" if link else ""
+        self._meta = None if meta is None else marshal_dumps(meta)
         self._hash = hash(
             (
                 self._color,
@@ -166,9 +174,10 @@ class Style:
                 self._attributes,
                 self._set_attributes,
                 link,
+                self._meta,
             )
         )
-        self._null = not (self._set_attributes or color or bgcolor or link)
+        self._null = not (self._set_attributes or color or bgcolor or link or meta)
 
     @classmethod
     def null(cls) -> "Style":
@@ -194,6 +203,7 @@ class Style:
         style._attributes = 0
         style._link = None
         style._link_id = ""
+        style._meta = None
         style._hash = hash(
             (
                 color,
@@ -342,9 +352,24 @@ class Style:
                 return value
         raise ValueError("expected at least one non-None style")
 
-    def __repr__(self) -> str:
-        """Render a named style differently from an anonymous style."""
-        return f'Style.parse("{self}")'
+    def __rich_repr__(self) -> RichReprResult:
+        yield "color", self.color, None
+        yield "bgcolor", self.bgcolor, None
+        yield "bold", self.bold, None,
+        yield "dim", self.dim, None,
+        yield "italic", self.italic, None
+        yield "underline", self.underline, None,
+        yield "blink", self.blink, None
+        yield "blink2", self.blink2, None
+        yield "reverse", self.reverse, None
+        yield "conceal", self.conceal, None
+        yield "strike", self.strike, None
+        yield "underline2", self.underline2, None
+        yield "frame", self.frame, None
+        yield "encircle", self.encircle, None
+        yield "link", self.link, None
+        if self._meta:
+            yield "meta", self.meta
 
     def __eq__(self, other: Any) -> bool:
         if not isinstance(other, Style):
@@ -355,6 +380,7 @@ class Style:
             and self._set_attributes == other._set_attributes
             and self._attributes == other._attributes
             and self._link == other._link
+            and self._meta == other._meta
         )
 
     def __hash__(self) -> int:
@@ -385,6 +411,15 @@ class Style:
         """A Style with background only."""
         return Style(bgcolor=self.bgcolor)
 
+    @property
+    def meta(self) -> Dict[str, Any]:
+        """Get meta information (can not be changed after construction)."""
+        return (
+            {}
+            if self._meta is None
+            else cast(Dict[str, Any], marshal_loads(self._meta))
+        )
+
     @property
     def without_color(self) -> "Style":
         """Get a copy of the style with color removed."""
@@ -401,6 +436,7 @@ class Style:
         style._link_id = f"{time()}-{randint(0, 999999)}" if self._link else ""
         style._hash = self._hash
         style._null = False
+        style._meta = None
         return style
 
     @classmethod
@@ -446,7 +482,7 @@ class Style:
         }
         color: Optional[str] = None
         bgcolor: Optional[str] = None
-        attributes: Dict[str, Optional[bool]] = {}
+        attributes: Dict[str, Optional[Any]] = {}
         link: Optional[str] = None
 
         words = iter(style_definition.split())
@@ -575,6 +611,7 @@ class Style:
         style._link_id = f"{time()}-{randint(0, 999999)}" if self._link else ""
         style._hash = self._hash
         style._null = False
+        style._meta = self._meta
         return style
 
     def update_link(self, link: Optional[str] = None) -> "Style":
@@ -597,6 +634,7 @@ class Style:
         style._link_id = f"{time()}-{randint(0, 999999)}" if link else ""
         style._hash = self._hash
         style._null = False
+        style._meta = self._meta
         return style
 
     def render(
@@ -657,6 +695,10 @@ class Style:
         new_style._link_id = style._link_id or self._link_id
         new_style._hash = style._hash
         new_style._null = self._null or style._null
+        if self._meta and style._meta:
+            new_style._meta = marshal_dumps({**self.meta, **style.meta})
+        else:
+            new_style._meta = self._meta or style._meta
         return new_style
 
 
